@@ -14,7 +14,8 @@ extern SEXP test_cox_tvc2(SEXP);
 extern SEXP test_cox_tvc2_grad(SEXP);
 extern SEXP fitCureModel(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
 extern SEXP aft_model_output(SEXP);
-extern SEXP vunirootRcpp(SEXP,SEXP,SEXP,SEXP,SEXP);
+extern SEXP vunirootRcpp(SEXP,SEXP,SEXP,SEXP,SEXP,SEXP,SEXP);
+extern SEXP multistate_ddt(SEXP,SEXP,SEXP,SEXP);
 
 /* .Fortran calls -- thanks to Gordon Smyth */
 extern void F77_NAME(gausq2)(void *, void *, void *, void *, void *);
@@ -25,7 +26,8 @@ static const R_CallMethodDef CallEntries[] = {
     {"test_cox_tvc2_grad", (DL_FUNC) &test_cox_tvc2_grad, 1},
     {"fitCureModel",       (DL_FUNC) &fitCureModel,       6},
     {"aft_model_output",   (DL_FUNC) &aft_model_output,   1},
-    {"vunirootRcpp",       (DL_FUNC) &vunirootRcpp,       5},
+    {"vunirootRcpp",       (DL_FUNC) &vunirootRcpp,       7},
+    {"multistate_ddt",     (DL_FUNC) &multistate_ddt,     4},
     {NULL, NULL, 0}
 };
 
